<?php
namespace app\common\plugin;
/**
 * Created by PhpStorm.
 * User: shaoguo
 * Date: 2019-02-26
 * Time: 12:57
 */

class Jssdk
{
    private $app_Id;
    private $app_Secret;

    public function __construct($str='mp_wxconfig') {
        $wxconfig=config($str);
        $this->app_Id =$wxconfig['appid'];
        $this->app_Secret =$wxconfig['secret'];
//        dump($wxconfig);
    }

    public function getSignPackage($url="") {
        $jsapiTicket = $this->getJsApiTicket();

        // 注意 URL 一定要动态获取，不能 hardcode.
        $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";

        $timestamp = time();
        $nonceStr = $this->createNonceStr();

        // 这里参数的顺序要按照 key 值 ASCII 码升序排序
        $string = "jsapi_ticket=$jsapiTicket&noncestr=$nonceStr&timestamp=$timestamp&url=$url";

        $signature = sha1($string);

        $signPackage = array(
            "appId"     => $this->app_Id,
            "nonceStr"  => $nonceStr,
            "timestamp" => $timestamp,
            "url"       => $url,
            "signature" => $signature,
            "rawString" => $string,
        	"ticket"	=>$jsapiTicket
        );
        return $signPackage;
    }

    private function createNonceStr($length = 16) {
        $chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        $str = "";
        for ($i = 0; $i < $length; $i++) {
            $str .= substr($chars, mt_rand(0, strlen($chars) - 1), 1);
        }
        return $str;
    }

    private function getJsApiTicket() {
        // jsapi_ticket 应该全局存储与更新，以下代码以写入到文件中做示例
    	$path=$this->app_Id."jsapi_ticket.json";
    	if(!file_exists($path)){
    		file_put_contents($path, '');
    		chmod($path, 0777);
    	}
        $data = json_decode(file_get_contents($path));
        $accessToken = $this->getAccessToken();
        // 如果是企业号用以下 URL 获取 ticket
        // $url = "https://qyapi.weixin.qq.com/cgi-bin/get_jsapi_ticket?access_token=$accessToken";
        $url = "https://api.weixin.qq.com/cgi-bin/ticket/getticket?type=jsapi&access_token=$accessToken";
        if(!empty($data)){
            if ($data->expire_time < time()) {

                $res = json_decode($this->httpGet($url));
                $ticket = $res->ticket;
                if ($ticket) {
                    $data->expire_time = time() + 7000;
                    $data->jsapi_ticket = $ticket;
                    $fp = fopen($path, "w");
                    fwrite($fp, json_encode($data));
                    fclose($fp);
                }
            } else {
                $ticket = $data->jsapi_ticket;
            }

        }else{
            $res = json_decode($this->httpGet($url));
            $ticket = $res->ticket;
            if ($ticket) {
                $res->expire_time = time() + 7000;
                $res->jsapi_ticket = $ticket;
                $fp = fopen($path, "w");
                fwrite($fp, json_encode($res));
                fclose($fp);
            }
        }

        return $ticket;
    }

	public function getAccessToken($refresh=false) {
        // access_token 应该全局存储与更新，以下代码以写入到文件中做示例
    	$path=$this->app_Id."access_token.json";
    	if(!file_exists($path)){
    		file_put_contents($path, '');
    		chmod($path, 0777);
    	}
        $data = json_decode(file_get_contents($path));
        $url = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=$this->app_Id&secret=$this->app_Secret";
        if(!is_null($data)){
            if ($data->expire_time < time()||$refresh) {
                // 如果是企业号用以下URL获取access_token
                // $url = "https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid=$this->app_Id&corpsecret=$this->app_Secret";
                $res = json_decode($this->httpGet($url));
                $access_token = $res->access_token;

                $data->expire_time = time() + 7000;
                $data->access_token = $access_token;

                $fp = fopen($path, "w");
                fwrite($fp, json_encode($data));
                fclose($fp);
            } else {
            	//检测是否失效
            	$test_url='https://api.weixin.qq.com/cgi-bin/get_current_selfmenu_info?access_token='.$data->access_token;
            	$res = json_decode($this->httpGet($test_url),true);
            	
            	if(isset($res['errcode'])){
            		$res = json_decode($this->httpGet($url));
            		$access_token = $res->access_token;
            		
            		$data->expire_time = time() + 7000;
            		$data->access_token = $access_token;
            		
            		$fp = fopen($path, "w");
            		fwrite($fp, json_encode($data));
            		fclose($fp);
            	}else{
                	$access_token = $data->access_token;
            	}
            }
        }else{
            $res = json_decode($this->httpGet($url));
            $access_token = $res->access_token;
            if ($access_token) {
                $res->expire_time = time() + 7000;
                $res->access_token = $access_token;
                $fp = fopen($path, "w");
                fwrite($fp, json_encode($res));
                fclose($fp);
            }

        }

        return $access_token;
    }

    private function httpGet($url) {
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 500);
        // 为保证第三方服务器与微信服务器之间数据传输的安全性，所有微信接口采用https方式调用，必须使用下面2行代码打开ssl安全校验。
        // 如果在部署过程中代码在此处验证失败，请到 http://curl.haxx.se/ca/cacert.pem 下载新的证书判别文件。
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
//        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, true);
        curl_setopt($curl, CURLOPT_URL, $url);

        $res = curl_exec($curl);
        curl_close($curl);
        return $res;
    }
}